from buptmw import BUPT_Auth
import requests

data = {
    "username": "2023213685",
    "password": "zero-bupt-A1"
}
BUPT_UCLOUD_CHECK = "https://apiucloud.bupt.edu.cn/blade-portal/home-page-info/getShufflingWebList"

cas = BUPT_Auth(cas=data)
ucloud = cas.get_Ucloud()

resp1 = ucloud.get(BUPT_UCLOUD_CHECK, headers={
    "blade-auth": ucloud.access_token
})
resp2 = requests.get(BUPT_UCLOUD_CHECK, headers={
    "blade-auth": ucloud.access_token
})
pass



from typing import Type, Optional
from urllib.parse import urlparse, parse_qs, quote
from base64 import b64encode
from datetime import datetime, timedelta

from buptmw.constants import BUPT_UCLOUD_LOGIN, BUPT_UCLOUD_TOKEN, BUPT_UCLOUD_INFO, BUPT_UCLOUD_CURRENT, BUPT_UCLOUD_USER, BUPT_UCLOUD_CHECK
from .cas import CAS


class Ucloud:
    class Exceptions:
        class NeedCAS(Exception):
            def __init__(self):
                self.message = "Need a BUPT.CAS to init."
                super().__init__(self.message)

    def __init__(self, cas: Optional[Type[CAS]] = None):
        if cas is None or not isinstance(cas, CAS):
            raise self.Exceptions.NeedCAS()
        self.cas = cas
        self._login()

    def check(self):
        resp = self.get(
            BUPT_UCLOUD_CHECK,
            headers={"blade-auth": self.access_token}
        )
        if resp.status_code == 200:
            return True
        return False

    def _get_cookies(self):
        info = self.cas.get(
            BUPT_UCLOUD_INFO,
            headers={
                "Authorization": self.authorization,
                "Blade-Auth": self.access_token
            }
        ).json()["data"]
        current = self.cas.get(
            BUPT_UCLOUD_CURRENT,
            headers={
                "Authorization": self.authorization,
                "Blade-Auth": self.access_token
            }
        ).json()["data"]
        user = self.cas.get(
            BUPT_UCLOUD_USER,
            headers={
                "Authorization": self.authorization,
                "Blade-Auth": self.access_token
            }
        ).json()["data"]

        cookies = {}
        cookies["iClass-uuid"] = self.user_id
        cookies["iClass-token"] = self.access_token
        cookies["iClass-refresh_token"] = self.refresh_token
        cookies["iClass-login-meth"] = "icloud"
        cookies["iClass-expert-account"] = self.user_name
        cookies["iClass-real_name"] = quote(self.real_name)
        cookies["iClass-role_name"] = self.role_name
        cookies["iClass-avatar"] = self.avatar
        cookies["iClass-loginId"] = self.loginId
        cookies["iClass-user-info"] = quote(str(info))
        cookies["iClass-current-term"] = quote(str(current))
        cookies["iClass-identity"] = self.identity
        cookies["iClass-user-role"] = quote(str(user[0]))
        cookies["iClass-login-roles"] = quote(str(user))

        for cookie in cookies:
            self.cas.session.cookies.set(
                name=cookie,
                value=cookies[cookie],
                domain="ucloud.bupt.edu.cn",
                expires=(datetime.now() + timedelta(hours=1)).timestamp()
            )
        pass

    def _login(self):
        self.authorization = "Basic " + b64encode("portal:portal_secret".encode()).decode()
        resp = self.cas.get(BUPT_UCLOUD_LOGIN)
        self.ticket = parse_qs(urlparse(resp.url).query)["ticket"][0]
        resp = self.cas.post(
            BUPT_UCLOUD_TOKEN,
            headers={
                "Authorization": self.authorization,
            },
            data={
                "ticket": self.ticket,
                "grant_type": "third"
            }
        )
        data = resp.json()
        self.user_id = data["user_id"]
        self.access_token = data["access_token"]
        self.refresh_token = data["refresh_token"]
        self.role_name = data["role_name"]
        self.loginId = data["loginId"]
        self.user_name = data["user_name"]
        self.real_name = data["real_name"]
        self.avatar = data["avatar"]
        self.dept_id = data["dept_id"]
        self.identity = f"{self.role_name}:{self.dept_id}"
        self._get_cookies()

    # redirect all undefined methods to self.cas
    def __getattr__(self, name):
        attr = getattr(self.cas, name)
        if callable(attr):
            return attr.__get__(self.cas, type(self.cas))
        else:
            return attr



# Session Bases
USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36"

# CAS
BUPT_CAS_LOGIN = "https://auth.bupt.edu.cn/authserver/login"

# UC
BUPT_UC_LOGIN = "https://uc.bupt.edu.cn/api/login?target=https://uc.bupt.edu.cn/#/user/login"
BUPT_UC_CHECK = "https://uc.bupt.edu.cn/api/uc/status"

# Ucloud
BUPT_UCLOUD_LOGIN = "https://auth.bupt.edu.cn/authserver/login?service=http://ucloud.bupt.edu.cn"
BUPT_UCLOUD_TOKEN = "https://apiucloud.bupt.edu.cn/ykt-basics/oauth/token"
BUPT_UCLOUD_INFO = "https://apiucloud.bupt.edu.cn/ykt-basics/info"
BUPT_UCLOUD_CURRENT = "https://apiucloud.bupt.edu.cn/ykt-site/base-term/current"
BUPT_UCLOUD_USER = "https://apiucloud.bupt.edu.cn/ykt-basics/userroledomaindept/listByUserId"
BUPT_UCLOUD_CHECK = "https://apiucloud.bupt.edu.cn/blade-portal/home-page-info/getShufflingWebList"



import json
import os


class ConfigItem:
    def __init__(self, config: dict):
        self.config = config
        for key in config:
            if not isinstance(self.config[key], dict):
                continue
            self.config[key] = ConfigItem(self.config[key])

    def __getattr__(self, name):
        return self.config.get(name, None)


class Config(ConfigItem):
    def __init__(self, folder_path):
        files = os.listdir(folder_path)
        config = {}
        for file in files:
            with open(os.path.join(folder_path, file)) as f:
                config[file.split(".")[0]] = ConfigItem(json.load(f))
        self.config = config




from typing import Type

from .plates.cas import CAS
from .plates.uc import UC
from .plates.ucloud import Ucloud


class BUPT_Auth:
    def __init__(self, cas=None):
        self._plates_instance = {}
        self._plates_map = {
            "UC": UC,
            "Ucloud": Ucloud
        }
        if cas is not None:
            self.cas = CAS(cas["username"], cas["password"])

    def _get_instance(self, label):
        plate = self._plates_map[label]
        if label not in self._plates_instance or not self._plates_instance[label].check():
            self._plates_instance[label] = plate(cas=self.cas)
        return self._plates_instance[label]

    def get_UC(self) -> Type[UC]:
        return self._get_instance("UC")

    def get_Ucloud(self) -> Type[Ucloud]:
        return self._get_instance("Ucloud")
